@@ -2,10 +2,12 @@ from __future__ import absolute_import
 
 import csv
 import datetime as dt
+from enum import Enum
 import json
 import io
 
 import arrow
+import openpyxl
 import pytest
 import xlrd
 import xlsxwriter
@@ -23,7 +25,7 @@ from webgrid import (
     col_styler,
     row_styler,
 )
-from webgrid.filters import TextFilter
+from webgrid.filters import TextFilter, OptionsEnumFilter
 from webgrid.renderers import (
     CSV,
     HTML,
@@ -51,6 +53,14 @@ from webgrid_ta.model.entities import (
 from .helpers import eq_html, inrequest, render_in_grid
 
 
+def _query_exclude_person(query):
+    # this is pretty limited, but only used in the below couple of grids to
+    # exclude the third Person record
+    persons = Person.query.order_by(Person.id).limit(3).all()
+    exclude_id = persons[2].id if len(persons) >= 3 else -1
+    return query.filter(Person.id != exclude_id)
+
+
 class PeopleGrid(PG):
     def query_prep(self, query, has_sort, has_filters):
         query = PG.query_prep(self, query, True, True)
@@ -61,7 +71,7 @@ class PeopleGrid(PG):
 
         # default filter
         if not has_filters:
-            query = query.filter(Person.id != 3)
+            query = _query_exclude_person(query)
 
         return query
 
@@ -78,7 +88,7 @@ class PeopleCSVGrid(PG):
 
         # default filter
         if not has_filters:
-            query = query.filter(Person.id != 3)
+            query = _query_exclude_person(query)
 
         return query
 
@@ -213,20 +223,24 @@ class TestHtmlRenderer(object):
     def test_car_html(self):
         key_data = (
             {'id': 1, 'make': 'ford', 'model': 'F150&', 'color': 'pink',
-             'dealer': 'bob', 'dealer_id': '7', 'active': True},
+             'dealer': 'bob', 'dealer_id': '7', 'active': True,
+             'active_1': True, 'active_2': True},
             {'id': 2, 'make': 'chevy', 'model': '1500', 'color': 'blue',
-             'dealer': 'fred', 'dealer_id': '9', 'active': False},
+             'dealer': 'fred', 'dealer_id': '9', 'active': False,
+             'active_1': False, 'active_2': False},
         )
 
         mg = CarGrid()
         mg.set_records(key_data)
         eq_html(mg.html.table(), 'basic_table.html')
 
+    @pytest.mark.skipif(db.engine.dialect.name != 'sqlite', reason="IDs will not line up")
     @inrequest('/')
     def test_people_html(self):
         pg = render_in_grid(PeopleGrid, 'html')()
         eq_html(pg.html.table(), 'people_table.html')
 
+    @pytest.mark.skipif(db.engine.dialect.name != 'sqlite', reason="IDs will not line up")
     @inrequest('/')
     def test_stopwatch_html(self):
         # Test Stopwatch grid with column groups.
@@ -380,7 +394,7 @@ class TestHtmlRenderer(object):
     def test_paging_html(self):
         g = self.get_grid()
         input_html = g.html.paging_input()
-        assert_tag(input_html, 'input', name='perpage', type='text', value='1')
+        assert_tag(input_html, 'input', name='perpage', type='number', value='1')
 
         img_html = g.html.paging_img_first()
         assert_tag(img_html, 'img', alt='<<', width='16', height='13',
@@ -516,6 +530,35 @@ class TestHtmlRenderer(object):
         filter_html = g.html.filtering_table_row(g.key_column_map['firstname'])
         assert '<tr class="firstname_filter" data-special-attr="foo">' in filter_html, filter_html
 
+    @inrequest('/thepage')
+    def test_filter_primary_op_specified(self):
+        g = PeopleGrid()
+        g.key_column_map['firstname'].filter.primary_op = '!eq'
+        filter_html = g.html.filtering_table_row(g.key_column_map['firstname'])
+        assert PyQuery(filter_html).find('option[value="!eq"]').attr('data-render') == 'primary'
+        assert not PyQuery(filter_html).find('option[value="eq"]').attr('data-render')
+
+    @inrequest('/thepage')
+    def test_filter_primary_op_not_specified(self):
+        g = PeopleGrid()
+        g.key_column_map['firstname'].filter.primary_op = None
+        filter_html = g.html.filtering_table_row(g.key_column_map['firstname'])
+        assert PyQuery(filter_html).find('option[value="eq"]').attr('data-render') == 'primary'
+        assert not PyQuery(filter_html).find('option[value="!eq"]').attr('data-render')
+
+    @inrequest('/thepage')
+    def test_multiselect_enum_options(self):
+        class PeopleType(Enum):
+            bob = 'Bob'
+            charlie = 'Charlie'
+
+        g = PeopleGrid()
+        name_filter = OptionsEnumFilter(Person.firstname, enum_type=PeopleType).new_instance()
+        name_filter.set('is', ['bob'])
+        output = g.html.filtering_filter_options_multi(name_filter, 'foo')
+        assert PyQuery(output).find('input[value="bob"]').attr('checked')
+        assert not PyQuery(output).find('input[value="charlie"]').attr('checked')
+
     @inrequest('/thepage')
     def test_confirm_export(self):
         g = PeopleGrid()
@@ -580,6 +623,19 @@ class TestHtmlRenderer(object):
         filter_html = g.html.filtering_fields()
         assert_tag(filter_html, 'input', id='search_input', name='search', type='text', value='foo')
 
+    @inrequest('/thepage?search=foo&dgreset=1')
+    def test_render_search_filter_reset(self):
+        g = PeopleGrid()
+        g.enable_search = True
+
+        filter_html = g.html.filtering_fields()
+        tag = assert_tag(filter_html, 'input', id='search_input', name='search', type='text')
+        assert tag.val() == ''
+
+        g.apply_qs_args()
+        filter_html = g.html.filtering_fields()
+        assert_tag(filter_html, 'input', id='search_input', name='search', type='text', value='')
+
     def test_search_disabled(self):
         class PeopleGrid2(PeopleGrid):
             enable_search = False
@@ -761,39 +817,40 @@ class TestXLSXRenderer(object):
         wb = g.xlsx()
         wb.filename.seek(0)
 
-        book = xlrd.open_workbook(file_contents=wb.filename.getvalue())
-        sh = book.sheet_by_name('render_in_grid')
+        book = openpyxl.load_workbook(wb.filename)
+        sh = book['render_in_grid']
+
         # headers
-        assert sh.ncols == 10
-        assert sh.cell_value(0, 0) == 'First Name'
-        assert sh.cell_value(0, 7) == 'State'
+        assert sh.max_column == 10
+        assert sh.cell(1, 1).value == 'First Name'
+        assert sh.cell(1, 8).value == 'State'
 
         # last data row
-        assert sh.cell_value(3, 0) == 'fn001'
-        assert sh.cell_value(3, 7) == 'st001'
-        assert sh.nrows == 4
+        assert sh.max_row == 4
+        assert sh.cell(4, 1).value == 'fn001'
+        assert sh.cell(4, 8).value == 'st001'
 
     def test_group_headings(self):
         grid = StopwatchGrid()
         wb = grid.xlsx()
         wb.filename.seek(0)
 
-        book = xlrd.open_workbook(file_contents=wb.filename.getvalue())
-        sheet = book.sheet_by_index(0)
-        # A [ 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 ]
-        #   [       | Lap 1 |   | Lap 2 | Lap 3 ]
-        assert sheet.row_values(0) == ['', '', 'Lap 1', '', '', 'Lap 2', '', 'Lap 3', '']
-        assert sheet.row_values(0) == ['', '', 'Lap 1', '', '', 'Lap 2', '', 'Lap 3', '']
-        assert sheet.cell_value(1, 1) == 'Label'
-        assert sheet.cell_value(2, 1) == 'Watch 1'
-        assert sheet.merged_cells == [
-            (0, 1, 0, 2),  # Buffer A1:A2
-            (0, 1, 2, 4),  # Lap 1  A3:A4
-            #              # Buffer A5
-            (0, 1, 5, 7),  # Lap 2  A6:A7
-            (0, 1, 7, 9),  # Lap 3  A8:A9
+        book = openpyxl.load_workbook(wb.filename)
+        sheet = book[book.sheetnames[0]]
+        #   [ A | B | C | D | E | F | G | H | I ]
+        # 1 [       | Lap 1 |   | Lap 2 | Lap 3 ]
+        row_values = [cell.value for cell in next(sheet.iter_rows(max_row=1))]
+        assert row_values == [None, None, 'Lap 1', None, None, 'Lap 2', None, 'Lap 3', None]
+        assert sheet.cell(2, 2).value == 'Label'
+        assert sheet.cell(3, 2).value == 'Watch 1'
+        assert [str(range_) for range_ in sheet.merged_cells.ranges] == [
+            'A1:B1',
+            'C1:D1',
+            # E is a single cell
+            'F1:G1',
+            'H1:I1',
         ]
-        assert sheet.ncols == 9
+        assert sheet.max_column == 9
 
     def test_subtotals_with_no_records(self):
         g = PGGrandTotals()
@@ -809,8 +866,8 @@ class TestXLSXRenderer(object):
         wb = g.xlsx()
         wb.filename.seek(0)
 
-        book = xlrd.open_workbook(file_contents=wb.filename.getvalue())
-        book.sheet_by_name('people_grid_with_a_really_r...')
+        book = openpyxl.load_workbook(wb.filename)
+        assert book['people_grid_with_a_really_r...']
 
     def test_totals(self):
         g = PeopleGrid()
@@ -819,12 +876,12 @@ class TestXLSXRenderer(object):
         wb = g.xlsx()
         wb.filename.seek(0)
 
-        book = xlrd.open_workbook(file_contents=wb.filename.getvalue())
-        sheet = book.sheet_by_index(0)
-        assert sheet.nrows == 5
-        assert sheet.cell_value(4, 0) == 'Totals (3 records):'
-        assert sheet.cell_value(4, 8) == 6.39
-        assert sheet.merged_cells == [(4, 5, 0, 8)]
+        book = openpyxl.load_workbook(wb.filename)
+        sheet = book[book.sheetnames[0]]
+        assert sheet.max_row == 5
+        assert sheet.cell(5, 1).value == 'Totals (3 records):'
+        assert sheet.cell(5, 9).value == 6.39
+        assert [str(range_) for range_ in sheet.merged_cells.ranges] == ['A5:H5']
 
     def test_totals_no_merge(self):
         class TestGrid(Grid):
@@ -836,12 +893,12 @@ class TestXLSXRenderer(object):
         wb = g.xlsx()
         wb.filename.seek(0)
 
-        book = xlrd.open_workbook(file_contents=wb.filename.getvalue())
-        sheet = book.sheet_by_index(0)
+        book = openpyxl.load_workbook(wb.filename)
+        sheet = book[book.sheetnames[0]]
 
-        assert sheet.nrows == 6
-        assert sheet.cell_value(5, 0) == 'Totals (4 records):'
-        assert sheet.merged_cells == []
+        assert sheet.max_row == 6
+        assert sheet.cell(6, 1).value == 'Totals (4 records):'
+        assert sheet.merged_cells.ranges == []
 
     def test_can_render(self):
         class FakeCountsGrid(PeopleGrid):
